@@ -34,13 +34,55 @@ def _interactive_display(filename):
     return in_notebook
 
 
-def _make_pretty_ticks(x_min, x_max, xs):
+def _make_pretty_ticks(x_min, x_max, x_axis_is_time):
 
-    x_ticks = [x_min, x_max]
+    if x_axis_is_time:
+        x_ticks = [datetime.fromtimestamp(_xt) for _xt in [x_min, x_max]]
+    else:
+        # TODO: if small number of x and equally spaced, just use them
+        min_rounded = '{:.1e}'.format(x_min)
+        max_rounded = '{:.1e}'.format(x_max)
+
+        x_ticks = [float(min_rounded)]
+        if min_rounded == max_rounded:
+            if x_max > float(max_rounded):
+                significand_st, exp_st = max_rounded.split("e")
+                max_out = float(str(.1) + 'e' + exp_st) + float(max_rounded)
+                x_ticks.append(max_out)
+            if (x_min < float(min_rounded)) or (x_max == float(max_rounded)):
+                significand_st, exp_st = min_rounded.split("e")
+                min_out = float(str(-.1) + 'e' + exp_st) + float(min_rounded)
+                x_ticks.insert(0, min_out)
+        else:
+            # TODO: add in more ticks if available
+            x_ticks.append(float(max_rounded))
 
     return x_ticks
 
 
+def _strip_zeros(x_tick):
+    if int(x_tick) == float(x_tick):
+        x_out = str(int(x_tick))
+    else:
+        x_out = str(x_tick)
+    return x_out
+
+
+def _num2pretty_string(x_tick):
+    if abs(x_tick) < 1000:
+        x_out = _strip_zeros(x_tick)
+    elif abs(x_tick) < 1000000:
+        x_out = _strip_zeros(x_tick/1000) + 'K'
+    elif abs(x_tick) < 1000000000:
+        x_out = _strip_zeros(x_tick/1000000) + 'M'
+    elif abs(x_tick) < 1000000000000:
+        x_out = _strip_zeros(x_tick/1000000000) + 'B'
+    else:
+        x_out = "{:E}".format(x_tick)
+
+    return x_out
+
+
 def _remove_extra_whitespace(in_string):
     in_string = ' '.join(in_string.split())
     return in_string
@@ -82,7 +124,9 @@ def _convert_to_lists_of_lists(xs, ys, yus, yls):
             yls = [yls]  # convert to list of lists
 
     xs = _convert_from_np_pd(xs)
-    if not all(isinstance(_x, list) for _x in xs):
+    if xs is None:
+        xs = [list(range(len(_y))) for _y in ys]  # convert to list of lists
+    elif not all(isinstance(_x, list) for _x in xs):
         xs = [xs for i in range(len(ys))]  # convert to list of lists
 
     return xs, ys, yus, yls
@@ -162,23 +206,24 @@ def _make_lines_and_labels(xs, ys, x2vb, y2vb, colors, labels, label_nudges,
                            tick_length, line_widths, points_radii):
     datalines = []
     all_circles = []
-    for line_ind, x in enumerate(xs):
-        y = ys[line_ind]
+    if ys != []:
+        for line_ind, y in enumerate(ys):
+            x = xs[line_ind]
 
-        assert len(x) == len(y), 'all xs and ys should be the same length'
+            assert len(x) == len(y), 'all xs and ys should be the same length'
 
-        x_vb = [x2vb(xi) for xi in x]
-        y_vb = [y2vb(yi) for yi in y]
-        dataline = ''
-        circles = '<g fill="{cc}" stroke="none" stroke-width="0">'
-        for xy_vb in zip(x_vb, y_vb):
-            dataline += str(xy_vb[0]) + ',' + str(xy_vb[1]) + ' '
-            circles += f'<circle cx="{xy_vb[0]}" cy="{xy_vb[1]}"'
-            circles += ' r="{cr}"/>'
+            x_vb = [x2vb(xi) for xi in x]
+            y_vb = [y2vb(yi) for yi in y]
+            dataline = ''
+            circles = '<g fill="{cc}" stroke="none" stroke-width="0">'
+            for xy_vb in zip(x_vb, y_vb):
+                dataline += str(xy_vb[0]) + ',' + str(xy_vb[1]) + ' '
+                circles += f'<circle cx="{xy_vb[0]}" cy="{xy_vb[1]}"'
+                circles += ' r="{cr}"/>'
 
-        circles += '</g>'
-        datalines.append(dataline)
-        all_circles.append(circles)
+            circles += '</g>'
+            datalines.append(dataline)
+            all_circles.append(circles)
 
     polylines = ''
     line_labels = '<g font-family="sans-serif" font-size="10" >'
@@ -254,6 +299,7 @@ def _make_y_axis(y_ticks, y_ticks_text, y_label, vb_width, vb_height, XBUF,
         for yt_ind, yt in enumerate(y_ticks[:-1]):
             yt_vb = y2vb(yt)
             ytn_vb = y2vb(y_ticks[yt_ind + 1])
+            yt_text = y_ticks_text[yt_ind]
             y_axis_pts_vb += _remove_extra_whitespace(f"""\
                 {y_axis_xt_vb},{yt_vb}
                 {y_axis_x_vb},{yt_vb}
@@ -261,12 +307,13 @@ def _make_y_axis(y_ticks, y_ticks_text, y_label, vb_width, vb_height, XBUF,
             y_axis_text_vb += '\n    '
             y_axis_text_vb += _remove_extra_whitespace(f"""\
                 <text x="{y_axis_x_vb - tick_length}" y="{yt_vb}" fill="black"
-                text-anchor="end" dominant-baseline="middle" > {yt} </text>""")
+                text-anchor="end" dominant-baseline="middle" > {yt_text}
+                </text>""")
 
         y_axis_pts_vb += f"""{y_axis_xt_vb},{ytn_vb}"""
         y_axis_text_vb += _remove_extra_whitespace(f"""\
             <text x="{y_axis_x_vb - tick_length}" y="{ ytn_vb}" fill="black"
-            text-anchor="end" dominant-baseline="middle" > {y_ticks[-1]}
+            text-anchor="end" dominant-baseline="middle" > {y_ticks_text[-1]}
             </text> </g>""")
         y_axis = _remove_extra_whitespace(f"""\
             <polyline fill="none" stroke="black" stroke-width="1"
@@ -303,10 +350,15 @@ def base_plot(xs, ys, yus=None, yls=None, filename='plot.svg',
               colors=None, fill_colors=None, fill_opacities=None,
               line_widths='1', points_radii=None,
               labels=None, label_nudges=None,
-              x_ticks=None, y_ticks=None, interactive_mode=True):
+              x_ticks=None, y_ticks=None, x_ticks_text=None,
+              interactive_mode=True):
     if filename[-4:] != '.svg':
         filename += '.svg'
 
+    if x_ticks_text is not None:
+        assert x_ticks is not None, 'x_ticks_text requies x_ticks'
+        assert len(x_ticks) == len(x_ticks_text), 'x_ticks x_ticks_text '
+
     XBUF = 0.1
     YBUF = 0.13
     tick_length = 2
@@ -316,8 +368,6 @@ def base_plot(xs, ys, yus=None, yls=None, filename='plot.svg',
     vb_width_in = vb_width*.0096*2
     vb_height_in = vb_height*.0096*2
 
-    xs, ys, yus, yls = _convert_to_lists_of_lists(xs, ys, yus, yls)
-
     if label_nudges is None and labels is not None:
         label_nudges = [0 for y in ys]
 
@@ -365,18 +415,18 @@ def base_plot(xs, ys, yus=None, yls=None, filename='plot.svg',
     y_max = max([max(y) for y in all_ys])
 
     if x_ticks is None:
-        x_ticks = _make_pretty_ticks(x_min, x_max, xs)
-        if x_axis_is_time:
-            x_ticks = [datetime.fromtimestamp(_xt) for _xt in x_ticks]
+        x_ticks = _make_pretty_ticks(x_min, x_max, x_axis_is_time)
 
-    x_ticks_text = [str(_xt) for _xt in x_ticks]
+    if not x_axis_is_time and x_ticks_text is None:
+        x_ticks_text = [_num2pretty_string(_xt) for _xt in x_ticks]
     if x_axis_is_time and len(x_ticks) >= 2:
-        x_ticks_text = [str(_xt).split(' ')[0] for _xt in x_ticks]
+        if x_ticks_text is None:
+            x_ticks_text = [str(_xt).split(' ')[0] for _xt in x_ticks]
         # trying to convert ticks
         x_ticks = [datetime.timestamp(_xt) for _xt in x_ticks]
 
     if y_ticks is None:
-        y_ticks = _make_pretty_ticks(y_min, y_max, ys)
+        y_ticks = _make_pretty_ticks(y_min, y_max, False)
 
     if type(labels) == str and len(ys) == 1:
         labels = [labels]  # convert to list with one string
@@ -414,7 +464,7 @@ def base_plot(xs, ys, yus=None, yls=None, filename='plot.svg',
                                                         x_label, vb_width,
                                                         vb_height, YBUF,
                                                         tick_length, x2vb)
-    y_ticks_text = []  # TODO
+    y_ticks_text = [_num2pretty_string(_yt) for _yt in y_ticks]
     y_axis, y_axis_text_vb, y_axis_label = _make_y_axis(y_ticks, y_ticks_text,
                                                         y_label, vb_width,
                                                         vb_height, XBUF,
@@ -493,8 +543,19 @@ def plot(*args, **kwargs):
     Tries to infer correct behavior when input is unexpected.
 
     """
+    if len(args) == 1:
+        xs = None
+        ys = args[0]
+    elif len(args) == 2:
+        xs = args[0]
+        ys = args[1]
 
-    return base_plot(*args, **kwargs)
+    yus = kwargs.pop('yus', None)
+    yls = kwargs.pop('yls', None)
+
+    xs, ys, yus, yls = _convert_to_lists_of_lists(xs, ys, yus, yls)
+
+    return base_plot(xs, ys, yus=yus, yls=yls, **kwargs)
 
 
 def scatter(*args, **kwargs):
@@ -548,7 +609,19 @@ def scatter(*args, **kwargs):
     if 'points_radii' not in kwargs.keys():
         kwargs['points_radii'] = [1]
 
-    return base_plot(*args, **kwargs)
+    if len(args) == 1:
+        xs = None
+        ys = args[0]
+    elif len(args) == 2:
+        xs = args[0]
+        ys = args[1]
+
+    yus = kwargs.pop('yus', None)
+    yls = kwargs.pop('yls', None)
+
+    xs, ys, yus, yls = _convert_to_lists_of_lists(xs, ys, yus, yls)
+
+    return base_plot(xs, ys, yus=yus, yls=yls, **kwargs)
 
 
 def error_plot(*args, **kwargs):
@@ -731,3 +804,52 @@ def _make_histogram_xys(hist_data, bin_edges):
     yl_out = [0 for y_o in y_out]
 
     return x_out, y_out, yl_out
+
+
+def bar(*args, bar_width=None, **kwargs):
+    """testing
+
+    """
+    if len(args) == 1:
+        autoposition_bars = True
+        xs = None
+        ys = args[0]
+    elif len(args) == 2:
+        autoposition_bars = False
+        xs = args[0]  # TODO check if xs is list of strings
+        ys = args[1]
+        if bar_width is None:
+            bar_width = 0.8
+
+    yus_in = kwargs.pop('yus', None)
+    yls_in = kwargs.pop('yls', None)
+    x_ticks = kwargs.pop('x_ticks', None)
+    fill_opacities = kwargs.pop('fill_opacities', [1])
+
+    xs, ys, _yus, _yls = _convert_to_lists_of_lists(xs, ys, None, None)
+
+    if autoposition_bars and (bar_width is None):
+        bar_width = 0.8 / len(xs)
+
+    yus, yls = [], []
+    for y_ind, y in enumerate(ys):
+        yu = [yi*_i for yi in y for _i in (0, 1, 1, 0)]
+        yl = [0 for yi in y for _i in (0, 1, 1, 0)]
+        yus.append(yu)
+        yls.append(yl)
+
+    xts = set()
+    xs_out = []
+    for x_ind, x in enumerate(xs):
+        bar_center = (-.5 * (len(xs) - 1) + x_ind) * bar_width
+        nudges = [-.5*bar_width, -.5*bar_width, .5*bar_width, .5*bar_width]
+        if autoposition_bars:
+            nudges = [bar_center + _n for _n in nudges]
+            xts = xts.union(set(x))
+        xs_out.append([xi+_d for xi in x for _d in nudges])
+
+    if x_ticks is None and autoposition_bars:
+        x_ticks = list(xts)
+
+    return base_plot(xs_out, ys=[], yus=yus, yls=yls, x_ticks=x_ticks,
+                     fill_opacities=fill_opacities, **kwargs)
